/*
 * analyzer.c
 *                   Simple analyzer for nova development
 *                             Masaharu Nomachi (KEK)
 *
 *              version 0.3   22-JUN-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.01  26-SEP-1992 M. Nomachi (KEK)
 *              version 1.10   2-Jul-1994 Yasuo TAKEUCHI     (T.I.T.)
 *
 * usage: test_analyzer [name [priority [seep_time]]]
 */
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "nova.h"

#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      10
#define DEFAULT_SLEEP    1

main(argc, argv) int argc; char *argv[]; {
    struct NOVAD *nd, *nova_copen();
    char name[NV_MAXNAME + 1];
    int *data1,*data2, length1, length2;
    int priority;
    int status, loop, i, sleep_time;
    int get_count, skip_count;
  
    /* Read options */
    strcpy(name, (argc > 1) ? argv[1] : DEFAULT_NAME);
    priority   = (argc > 2) ? atoi(argv[2]) : DEFAULT_PRI;
    sleep_time = (argc > 3) ? atoi(argv[3]) : DEFAULT_SLEEP;
  
    /* open NOVA */
    if ((int)(nd = nova_open(name, priority,2)) == -1) {
	printf("analyzer: cannot open %s", name);
	exit(1);
    }
  
    loop = 0;
    while(1) {
	status = nova_get(nd, &data1, &length1);
	switch (status) {
	  case 0:   /* nomal */
	    loop = data1[0];
	    get_count++;
	    printf("analyzer1: count=%d  length=%d byte", get_count, length1);
	    for(i = 0; i < 3; i++) printf(" %8X", data1[i]);
	    printf("\n");
	    /*******************************/
	    getchar();                /* */
	    /*******************************/

	    status = nova_get(nd, &data2, &length2);
	    switch (status) {
	      case 0:   /* nomal */
		loop = data2[0];
		get_count++;
		printf("analyzer2: count=%d  length=%d byte",
		       get_count, length2);
		for(i = 0; i < 3; i++) printf(" %8X", data2[i]);
		printf("\n");
		/*******************************/
		getchar();                /* */
		/*******************************/

		status = nova_put(nd, data2, 0, 0);
		printf(" put buffer 2\n");
		getchar();                /* */

		status = nova_put(nd, data1, 0, 0);
		printf(" put buffer 1\n");
		getchar();                /* */

		break;
	      case 1:   /* Interrupted */
		printf("analyzer: interrupted!\n");
		get_count = 0;
		skip_count = 0;
		loop = 0;
		break;
	      default:  /* error */
		printf("analyzer: error! %d\n",status);
		break;
	    }
	    
	    break;
	  case 1:   /* Interrupted */
	    printf("analyzer: interrupted!\n");
	    get_count = 0;
	    skip_count = 0;
	    loop = 0;
	    break;
	  default:  /* error */
	    printf("analyzer: error! %d\n",status);
	    break;
	}
	if (status < 0)  break;
    }
  
    printf("analyzer: nomal end.\n");
    nova_close(nd);
}

