/*
 * analyzer_val.c
 *                 Simple analyzer for valiable nova development
 *              version 1.02  01-OCT-1992 M. Nomachi (KEK)
 *              version 1.03  11-JUN-1994 Y. Fukuda (ICRR)
 *
 * usage: analyzer_fast [priority [name]]]
 */
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <errno.h>
#include "nova.h"

#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      50
#define DEFAULT_LOOP     10

int buffer[500000];

main(argc, argv) int argc; char *argv[]; {
  struct NOVAD *nd, *nova_copen();
  struct timeval last, present;
  char name[NV_MAXNAME + 1];
  int *data, length, priority;
  int status, loop, i, sleep_time, total_size = 0;
  int get_count, skip_count, init = 0;
  int difftime = 0, max_loop;
  
  /* Read options */
  max_loop   = (argc > 1) ? atoi(argv[1]) : DEFAULT_LOOP;
  priority   = (argc > 2) ? atoi(argv[2]) : DEFAULT_PRI;
  strcpy(name, (argc > 3) ? argv[3] : DEFAULT_NAME);
  
  /* open NOVA */
  if ((int)(nd = nova_open(name, priority,1)) == -1) {
    printf("analyzer: cannot open %s", name);
    exit(1);
  }
  
  /* initialize counter */
  get_count = 0;
  skip_count = 0;
  loop = 0;

  while(1) {
    status = nova_get(nd, &data, &length);
    switch (status) {
      case 0:   /* nomal */
	loop += 1;
	/* start time if first */
	total_size += length;
	gettimeofday(&present, NULL);
	if(init == 0) {
	    last.tv_usec = present.tv_usec;
	    last.tv_sec = present.tv_sec;
	    init = 1;
	}
	else {
	    difftime = present.tv_usec - last.tv_usec;
	    if(present.tv_sec > last.tv_sec) 
	      difftime += 1000000*(present.tv_sec-last.tv_sec);
/*	    printf("analyzer: %6d byte total:%8d byte %9.1f byte/sec\n ", 
		   length, total_size, 
		   (double)(total_size)/(double)difftime*1000000.); */
	}
	/*
	   skip_count += (data[0]-loop-1);
	   loop = data[0];
	   */
	get_count++;
	/*
	   for(i = 0; i < (length>>2); i++) buffer[i]=data[i];
	   */
	status = nova_put(nd, data, 0, 0);
	sleep(1);
	break;
      case 1:   /* Interrupted */
	if (get_count+skip_count) 
	  printf("analyzer: interrupted! get %d / %d %d%%\n",
		 get_count, get_count+skip_count,
		 100*get_count/(get_count+skip_count));
	else
	  printf("analyzer: interrupted! get %d / %d \n",
		 get_count, get_count+skip_count);
	get_count = 0;
	skip_count = 0;
	loop = 0;
	break;
      default:  /* error */
	printf("analyzer: error! %d\n",status);
	break;
    }
    if (status < 0)  break;
/*    if(loop >= max_loop) break; */
}
  printf("analyzer: %6d byte total:%8d byte %9.1f loop/sec\n ", 
	 length, total_size, 
	 (double)(loop)/(double)difftime*1000000.);
  
  printf("analyzer: nomal end.\n");
  nova_close(nd);
}

