/*
 * nv_open.c
 *                 novasub rouitines
 *
 *              version 0.3   22-JUN-1992 Yasuo TAKEUCHI     (T.I.T.)
 *              version 1.01  25-SEP-1992 Masaharu NOMACHI    (KEK)
 *              version 1.04  27-OCT-1992 Masaharu NOMACHI    (KEK)
 */

#include "novalib.h"
#include "nova.h"

/**************************************
 * nova_open                          *
 *   positive priority : consumer     *
 *   negative priority : producer     *
 **************************************/
struct NOVAD *nova_open(name, priority,request)
     char *name; int priority, request;
{
  struct NOVAD *nd;
  int rval, sd;
  int pid, novaid, shmid;
  int snd_msgid, rcv_msgid, snd_type, rcv_type;
  char *base;
  struct sockaddr_un server;
  struct msgbuf *buffer;
  char *function;
  int nv_dumy();

  function = (char *)signal(SIGALRM,nv_dumy);
  if ((void (*)())function != SIG_DFL) signal(SIGALRM,function);

  
  /*
   * get nova descriptor area
   */
  if ((int)(nd = (struct NOVAD *)malloc(sizeof(struct NOVAD))) == 0)
    return nd;

  /*
   * get message buffer area
   */
  if ((int)(buffer = (struct msgbuf *)malloc(sizeof(long)+NV_MSGLEN)) == 0) {
    free(nd);
    return (struct NOVAD *)(-1);
  }
  nd->buffer = buffer;
  nd->sd     = 0;

  /*
   * create socket
   */
  if ((sd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
    perror("nv_open: opening stream socket");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  nd->sd = sd;

  /*
   * connect socket
   */
  server.sun_family = AF_UNIX;
  strcpy(server.sun_path, name);
  if (connect(sd, &server, sizeof(struct sockaddr_un)) < 0) {
    perror("nv_open: connecting stream socket");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  
  /*
   * send pid and priority to nova daemon
   */
  pid = getpid();
  nd->pid = pid;
  nd->priority = priority;
  nd->request  = request;
  if (nv_msgget(nd) < 0) {
    perror("nv_open: getting individual message queue");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  sprintf(buffer->mtext, "install:%d:%d:%d:%d:",
	  pid,priority,request,nd->rcv_msgid);
  if (write(sd, buffer->mtext, strlen(buffer->mtext)) < 0) {
    perror("nv_open: writing on stream socket");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }

  /*
   * Get novaid and msgid, shmid from nova daemon
   */
  if (read(sd, buffer->mtext, NV_MSGLEN) < 0) {
    perror("nv_open: reading from stream socket");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  rval = sscanf(buffer->mtext,"ack:%d:%d:%d:(%d,%d):(%d,%d):",
	     &pid,&novaid,&shmid,
	     &rcv_msgid,&rcv_type,&snd_msgid,&snd_type);
  if (rval != 7) {
    printf("nv_open: wrong message is returned %s\n", buffer->mtext);
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  if (pid != (nd->pid)) {
    printf("nv_open: wrong pid is returned %d/%d\n", pid, nd->pid);
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  if (novaid == 0) {
    printf("nv_open: invalid novaid\n");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  nd->novaid = novaid;
  nd->snd_msgid  = snd_msgid;
  nd->rcv_msgid  = rcv_msgid;
  nd->snd_type   = snd_type;
  nd->rcv_type   = rcv_type;
  
  /*
   * create IPC shared memory 
   */
  if ((int)(base = (char *)shmat(shmid, 0, 0)) == -1) {
    perror("nv_open: shared memory attatch");
    nova_close(nd);
    return (struct NOVAD *)(-1);
  }
  nd->shmid  = shmid;
  nd->base   = base;
/*  
  printf("nv_open: pid=%d  novaid=%d  sd=%d  shmid=%d \n",
	 nd->pid, nd->novaid, nd->sd, nd->shmid);
  printf("         send msgid=%d %d receive msgid=%d %d\n",
	 nd->snd_msgid, nd->snd_type, nd->rcv_msgid, nd->rcv_type);
*/
  return nd;
}

nv_msgget(nd) struct NOVAD *nd; {
#ifdef NV_INDIV_MQ
  if (((nd->rcv_msgid) = msgget(IPC_PRIVATE, IPC_CREAT|0666)) < 0) {
    nd->rcv_msgid = 0;
    return -1;
  }
#endif
  return 0;
}

nv_dumy() {
  return 0;
}
