/*
 * bm_var.c
 *
 * variable length buffer extension
 *
 *              version 1.10  14-Jul-1993 Yasuo TAKEUCHI     (T.I.T.)
 */
#include "bm.h"
#include "bm_exec.h"
#include "bm_config.h"

extern struct EXEC_process *consumer;
extern struct EXEC_process *wait_producer, *busy_producer;
extern struct EXEC_buffer  *free_buffer;
extern int threshold;

/* gloval */
int max_buffer_size;       /* maximum buffer size for nova_get @producer */
int max_buffer_size_org;   /* maximum buffer size for nova_get @producer */

/*************************************/
void put_free_buffer(buffer)
/*************************************/
struct EXEC_buffer *buffer;
{
    struct EXEC_buffer *buf;

    if (free_buffer == NULL_buffer) {
	free_buffer      = buffer;
	buffer->next     = buffer;
	buffer->previous = buffer;
    }
    else {
	if (free_buffer->offset > buffer->offset) {
	    /* insert top */
	    free_buffer->previous->next = buffer;
	    buffer->previous            = free_buffer->previous;
	    free_buffer->previous       = buffer;
	    buffer->next                = free_buffer;
	    free_buffer = buffer;
	}
	else {
	    buf = free_buffer->next;
	    while (buf->offset > 0 && buf->offset < buffer->offset && 
		   buf != free_buffer) {
		buf = buf->next;
	    }
	    /* insert free buffer */
	    buf->previous->next = buffer;
	    buffer->previous    = buf->previous;
	    buf->previous       = buffer;
	    buffer->next        = buf;
	}
    }
    return;
}

/*************************************/
void move_free_tail(buf)
/*************************************/
struct EXEC_buffer *buf;
{
    if (free_buffer == NULL_buffer)
        return;

    if (buf == free_buffer) {
	free_buffer = buf->next;
	return;
    }

    /* remove buffer */
    buf->next->previous = buf->previous;
    buf->previous->next = buf->next;

    /* insert buffer at tail */
    buf->next     = free_buffer;
    buf->previous = free_buffer->previous;
    free_buffer->previous->next = buf;
    free_buffer->previous       = buf;

    return;
}



/*
 * source => source  + dest (request_size)
 */
/*************************************/
void devide_buffer(source, dest, request_size)
/*************************************/
struct EXEC_buffer *source, *dest;
int request_size;
{
    /* devide source buffer */
    dest->type         = source->type;
    dest->significance = source->significance;
    dest->status       = source->status;
    dest->offset       = source->offset;
    dest->max_size     = request_size;
    dest->size         = request_size;
    source->offset     = source->offset + request_size;
    source->max_size   = source->max_size - request_size;
    source->size       = source->size - request_size;

    return;
}


/*
 *  not continuous     0
 *  buf1 => buf2       1
 *  buf2 => buf1       2
 */
/*************************************/
int check_merge_buffer(buf1, buf2)
/*************************************/
struct EXEC_buffer *buf1, *buf2;
{
    if (buf1->offset + buf1->max_size == buf2->offset) 
        return 1;
    if (buf2->offset + buf2->max_size == buf1->offset) 
        return 2;
    return 0;
}


/*
 * buf1 + buf2  => buf1
 */
/*************************************/
void merge_buffer(buf1, buf2)
/*************************************/
struct EXEC_buffer *buf1, *buf2;
{
    buf1->max_size = buf1->max_size + buf2->max_size;
    buf2->offset   = -1;
    buf2->max_size = 0;
    return;
}


/*************************************/
void copy_buffer(buf1, buf2)
/*************************************/
struct EXEC_buffer *buf1, *buf2;
{
    buf2->type         = buf1->type;
    buf2->significance = buf1->significance;
    buf2->status       = buf1->status;
    buf2->offset       = buf1->offset;
    buf2->size         = buf1->size;
    buf2->max_size     = buf1->max_size;
    return;
}


/*************************************/
void remove_free_buffer(buf)
/*************************************/
struct EXEC_buffer *buf;
{
    if (buf == free_buffer) {
        if (buf->next != free_buffer)
	    free_buffer = buf->next;
	else
	    free_buffer = NULL_buffer;
    }
    buf->next->previous = buf->previous;
    buf->previous->next = buf->next;
    buf->next = 0;
    buf->previous = 0;
    return;
}


/*************************************/
void remove_wait_buffer(buf,c)
/*************************************/
struct EXEC_process *c;
struct EXEC_buffer *buf;
{
    if (buf == c->wait) {
        if (buf->next != c->wait)
	    c->wait = buf->next;
	else
	    c->wait = NULL_buffer;
    }
    buf->next->previous = buf->previous;
    buf->previous->next = buf->next;
    buf->next = 0;
    buf->previous = 0;
    return;
}


/*****************************************************************/
/*************************************/
void append_free_buffer(buf)
/*************************************/
struct EXEC_buffer *buf;
{
    struct EXEC_buffer *b;

    if (buf == NULL_buffer)
        return;

    b = buf->next;
    while(b != buf) {
	/* remove buffer */
	buf->next->previous = buf->previous;
	buf->previous->next = buf->next;
	/* put free buffer queue */
	put_free_buffer(buf);
	/* next buffer */
	buf = b;
	b = buf->next;
    }
    put_free_buffer(buf);

    return;
}

/*************************************/
struct EXEC_buffer *get_wait_buffer(c)
/*************************************/
struct EXEC_process *c;
{
    struct EXEC_buffer  *b;
    
    b = NULL_buffer;
    if (c->wait != NULL_buffer) {
	b = c->wait;
	do {
	    if ((b->significance + c->priority) < threshold) {
		remove_wait_buffer(b, c);
		return b;
	        break;
	    }
	    b = b->next;
	} while (b != c->wait);
    }
    return NULL_buffer;
}


/*************************************/
int get_null_buffer(buf) 
/*************************************/
struct EXEC_buffer **buf;
{
    if (free_buffer == NULL_buffer)
        return 1;

    *buf = free_buffer->next;
    while ((*buf)->max_size != 0 && *buf != free_buffer)
        *buf = (*buf)->next;
    if ((*buf)->max_size == 0) {
        remove_free_buffer(*buf);
        return 0;
    }
    return 1;
}


/*************************************/
void swap_buffer(buf1, buf2)
/*************************************/
struct EXEC_buffer *buf1, *buf2;
{
    struct EXEC_buffer tmp;
    copy_buffer(buf1, &tmp);
    copy_buffer(buf2, buf1);
    copy_buffer(&tmp, buf2);
    return;
}
    

/*************************************/
int sort_2_buffer(buf1, buf2)
/*************************************/
struct EXEC_buffer *buf1, *buf2;
{
    if (buf1->offset < 0 && buf2->offset >= 0) {
        swap_buffer(buf1, buf2);
	return 1;
    }
    if (buf2->offset >= 0 && buf1->offset > buf2->offset) {
        swap_buffer(buf1, buf2);
	return 1;
    }
    return 0;
}


/*************************************/
void sort_free_buffer()
/*************************************/
{
    struct EXEC_buffer *buf1, *buf2, *b;
    int sort_flag = 1;

    while(sort_flag != 0) {
	sort_flag = 0;
	buf1 = free_buffer;
	buf2 = buf1->next;
#ifdef DEBUG
	print_free_buffer("sort");
#endif
	while (buf2 != free_buffer) {
	    sort_flag += sort_2_buffer(buf1, buf2);
	    buf1 = buf2;
	    buf2 = buf1->next;
	}
    }
    return;
}


/*************************************/
void condence_free_buffer()
/*************************************/
{
    struct EXEC_buffer *buf1, *buf2;

    if (free_buffer == NULL_buffer)
        return;

    /* sort free buffers */
/*    sort_free_buffer(); */

    /* merge free buffers, if possible */
    buf1 = free_buffer;
    buf2 = free_buffer->next;
    while(buf2->max_size > 0 && buf1->offset < buf2->offset) {
	if (check_merge_buffer(buf1, buf2) == 1) {
	    merge_buffer(buf1, buf2);
	    buf2 = buf2->next;
	    move_free_tail(buf2->previous);
	}
	else {
	    buf1 = buf2;
	    buf2 = buf1->next;
	}
#ifdef DEBUG
	print_free_buffer("sort");
#endif
    }
    return;
}


/*****************************************************************/
/*************************************/
struct EXEC_buffer *get_free_buffer() 
/*************************************/
{
    struct EXEC_buffer *buffer, *buf1;
    if (free_buffer == NULL_buffer)
        return NULL_buffer;

    condence_free_buffer();

    /* search a free buffer which size is larger than "max_buffer_size" */
    buffer = free_buffer;
    do {
	if (buffer->max_size >= max_buffer_size)
	    break;
	buffer = buffer->next;
    } while (buffer->max_size > 0 && buffer != free_buffer);

    /* found a larger size free buffer */
    if (buffer->max_size > max_buffer_size) {
	if (get_null_buffer(&buf1) == 0) {
	    devide_buffer(buffer, buf1, max_buffer_size);
	    return buf1;
	}
	else { 
	    remove_free_buffer(buffer);
	    return buffer;
	}
    }
    /* found a same size free buffer */
    if (buffer->max_size == max_buffer_size) {
	remove_free_buffer(buffer);
        return buffer;
    }

    /* there is no suitable free buffer */
    return NULL_buffer;
}


/*****************************************************************/

#ifdef DEBUG

/*************************************/
print_free_buffer(s)
/*************************************/
char *s;
{
    struct EXEC_buffer *b;
    int i, imax, id[10], off[10], size[10];
    b = free_buffer;
    id[0]   = b->table->id;
    off[0]  = b->offset;
    size[0] = b->max_size;

    for (imax = 1; imax < 10; imax++) {
	b = b->next;
	if (b == free_buffer)
	    break;
	id[imax]   = b->table->id;
	off[imax]  = b->offset;
	size[imax] = b->max_size;
    }

    printf("===== %s =====\n", s); 
    for (i = 0 ; i < imax; i++) printf("=>%5d ", id[i]);
    printf("\n");
    for (i = 0 ; i < imax; i++) printf("  %5d ", off[i]);
    printf("\n");
    for (i = 0 ; i < imax; i++) printf("  %5d ", size[i]);
    printf("\n\n");
}

print_buffer(b)
struct EXEC_buffer *b;
{
    printf("%d %d %d %d\n", 
	   b->table->id, b->offset, b->max_size, b->size);
}

#endif

